# Auto-generated program tcl file

open_hw
connect_hw_server
open_hw_target
current_hw_device [get_hw_devices {{ hw_device }}]
set_property PROGRAM.FILE { {{ bitstream_name }} } [get_hw_devices {{ hw_device }}]
program_hw_devices [get_hw_devices {{ hw_device }}]
